import 'package:zap_architecture/zap_architecture.dart';

/// Created by Musa Usman on 14.09.2020
/// Copyright © 2020 Musa Usman. All rights reserved.
///
/// Email: hello@musausman.com
/// Website: musausman.com
/// WhatsApp: +92 324 9066001

/// You can use this in your UI and show loading indicators dialogs and error message dialogs
/// based on the data here.
class Process {
  /// This variable is used to check the [Process] is currently running.
  bool working = false;

  /// This variable is used to check the [Process] finished with an error.
  bool error = false;

  /// This is the error message which contains specific details for the error.
  /// Like exception and stack trace e.t.c.
  /// This can be null if the [Process] finishes successfully.
  ///
  /// So make sure to check that [error] == true before reading this value.
  /// Like so:
  ///
  /// if (process.error) {
  ///   print("ERROR: ${process.errorMessage}");
  /// }
  String errorMessage;

  /// This is the error message you should show to your user, like
  /// "Internet Not Available" or "Plese check your internet connection and try
  /// again".
  ///
  /// Usage example in UI:
  ///
  /// if (process.error) {
  ///   print("ERROR: ${process.errorMessage}");
  ///
  ///   setState(() {
  ///     _userFriendlyErrorMessage = process.friendlyErrorMessage;
  ///   });
  /// }
  String friendlyErrorMessage;

  /// This is usually used as the default value for a [Process].
  /// [ProcessMixin.process] uses this as the default value.
  ///
  /// THIS SAYS:
  /// "I'm not working right now and I haven't encountered any issues."
  Process.clear() {
    this.working = false;
    this.error = false;
    this.errorMessage = null;
    this.friendlyErrorMessage = null;
  }

  /// This is used when starting a process.
  ///
  /// Quick usage Example:
  ///   process = Process.start();
  ///
  /// THIS SAYS:
  /// "I have started working but I have not encountered any issues so far."
  Process.start()
      : this.working = true,
        this.error = false,
        this.friendlyErrorMessage = null,
        this.errorMessage = null;

  /// This is used when stopping a process.
  ///
  /// Quick usage Example:
  ///   process = Process.stop(Res);
  ///
  /// THIS SAYS:
  /// "I have stopped working and this is the error I encountered (if any)."
  Process.stop(Res response)
      : this.working = false,
        this.error = response.success == false,
        this.friendlyErrorMessage =
            response.success == false ? response.friendlyErrorMessage : null,
        this.errorMessage =
            response.success == false ? response.errorMessage : null;

  /// Overriding this and manually printing the data makes it easier to debug.
  @override
  String toString() =>
      "Process(working: $working, error: $error, errorMessage: $errorMessage, friendlyErrorMessage: $friendlyErrorMessage)";
}
