/// Created by Musa Usman on 14.09.2020
/// Copyright © 2020 Musa Usman. All rights reserved.
///
/// Email: hello@musausman.com
/// Website: musausman.com
/// WhatsApp: +92 324 9066001

class Res<T> {
  /// This class stores any sort of data.
  ///
  /// This is very helpful when doing Async programming, when you want to find out what happened to your call.
  /// Was it successful? Were there any errors? What is the data? What are the error messages?
  ///
  /// I have implemented Dart Generics so that you can define the type of the
  /// data. Using this will make your code compile-safe so that you do not
  /// encounter any errors while the app is running.
  ///
  /// You can set your data type for a Res like so:
  ///
  /// Res<String> Res = await myAsyncFunction();
  ///
  /// If myAsyncFunction tries to return a Res<int>, the Dart Analyzer will
  /// let you know so that you can fix the issue before running the app.
  T data;

  /// A simple boolean to check if your process returned a successful Res or not.
  bool success;

  /// This is the error message which can contain the StackTrace and other error
  /// details that are used for debugging.
  String errorMessage;

  /// This is the error message that can be shown to users.
  String friendlyErrorMessage;

  /// Use this contructor to return data after a function is successfully
  /// executed.
  ///
  /// Your code should be like this:
  ///
  /// Res<String> getHelloMessage() {
  ///   return Res.success("Hello World!");
  /// }
  ///
  Res.success(this.data) : this.success = true;

  /// Use this contructor to return data after a function is successfully
  /// executed.
  ///
  /// Your code should be like this:
  ///
  /// Res<String> getHelloMessage() {
  ///   return Res.error(
  ///     errorMessage: "MESSAGE_NOT_AVAILABLE. THIS MSG IS FOR DEVELOPER.",
  ///     friendlyErrorMessage: "Sorry, the hello message is not available.",
  ///   );
  /// }
  ///
  Res.error(
      {this.errorMessage, this.friendlyErrorMessage = "Something Went Wrong."})
      : this.success = false;

  /// Use this contructor whe you want to parse some JSON data that is fetched
  /// from your backend.
  ///
  /// I'm a fullstack developer, so all my REST APIs have the same data
  /// structure. So this constructor works well in my case.
  ///
  /// Your code should be like this:
  ///
  /// Res<String> getHelloMessage() {
  ///
  ///   Map<String, dynamic> jsonAPIRes = {
  ///     "success": true,
  ///     "data": "Hello World!",
  ///   };
  ///
  ///   return Res<String>.fromJSON(jsonAPIRes);
  /// }
  ///
  Res.fromJSON(Map<String, dynamic> json)
      : this.success = json['success'],
        this.data = json['data'],
        this.errorMessage = json['error_message'],
        this.friendlyErrorMessage = json['friendly_error_message'];

  Map<String, dynamic> json() {
    assert(this.data is String ||
        this.data is int ||
        this.data is double ||
        this.data is Map ||
        this.data is List ||
        this.data is bool);

    return <String, dynamic>{
      "success": this.success,
      "data": this.data,
    };
  }
}
