import 'package:zap_architecture/src/res.dart';
import 'package:zap_architecture/zap_architecture.dart';

/// Created by Musa Usman on 14.09.2020
/// Copyright © 2020 Musa Usman. All rights reserved.
///
/// Email: hello@musausman.com
/// Website: musausman.com
/// WhatsApp: +92 324 9066001

/// Designed for use with [ChangeNotifier] and [Provider].
/// [Status] is used inside the [StatusMixin] which handles the boiler plate
/// when adding this to a [ChangeNotifier].
///
/// You can use this in your UI and show loading indicators and error messages
/// based on the data here.
class Status {
  /// This is used to check if the [Status] is NOT loading and NOT showing an error.
  ///
  /// This is true when data has been successfully loaded.
  /// Or loading has not been started.
  bool get isClear => !loading && !error;

  /// If the [Status] is loading some sort of data.
  bool loading = false;

  /// If the [Status] encountered some sort of error.
  bool error = false;

  /// This is the error message which contains specific details for the error.
  /// Like exception and stack trace e.t.c.
  /// This can be null if the [Status] is clear or if data loaded successfully.
  ///
  /// So make sure to check that [error] == true before reading this value.
  /// Like so:
  ///
  /// if (status.error) {
  ///   print("ERROR: ${status.errorMessage}");
  /// }
  String errorMessage;

  /// This is the error message you should show to your user, like
  /// "Internet Not Available" or "Plese check your internet connection and try
  /// again".
  ///
  /// Usage example in UI:
  ///
  /// if (status.error) {
  ///   print("ERROR: ${status.errorMessage}");
  ///
  ///   setState(() {
  ///     _userFriendlyErrorMessage = status.friendlyErrorMessage;
  ///   });
  /// }
  String friendlyErrorMessage;

  /// This is usually used as the default value for a [Status].
  /// [StatusMixin.status] uses this as the default value.
  ///
  /// THIS SAYS:
  /// "I'm not loading data right now and I haven't encountered any errors."
  Status.clear() {
    this.loading = false;
    this.error = false;
    this.errorMessage = null;
    this.friendlyErrorMessage = null;
  }

  /// This is used when the status is set to loading state.
  ///
  /// Quick usage Example:
  ///   status = Status.loading();
  ///
  /// THIS SAYS:
  /// "I have started loading but I have not encountered any errors so far."
  Status.loading()
      : this.loading = true,
        this.error = false,
        this.friendlyErrorMessage = null,
        this.errorMessage = null;

  Status.error([this.friendlyErrorMessage = "Something Went Wrong."])
      : this.loading = false,
        this.error = true;

  /// This is used when finishing up a loading function and the data is being
  /// fetched in a [Res] class.
  ///
  /// Quick usage Example:
  ///   status = Status.fromRes(response);
  ///
  /// THIS SAYS:
  /// "I have finished loading and this is the error I encountered (if any)."
  Status.fromResponse(Res response)
      : this.loading = false,
        this.error = response.success == false,
        this.friendlyErrorMessage = (response.success == false) == true
            ? response.friendlyErrorMessage
            : null,
        this.errorMessage =
            (response.success == false) == true ? response.errorMessage : null;

  /// Overriding this and manually printing the data makes it easier to debug.
  @override
  String toString() =>
      "Status(loading: $loading, error: $error, errorMessage: $errorMessage, friendlyErrorMessage: $friendlyErrorMessage)";
}
